from pystencilssfg.lang import CppClass, AugExpr
from pystencilssfg.lang.types import cpptype
from pystencilssfg.ir import SfgClass


class GeneratedClassWrapperBase(CppClass):
    _class: SfgClass
    _namespace: str | None

    def __init_subclass__(cls) -> None:
        typename = (
            f"{cls._namespace}::{cls._class.class_name}"
            if cls._namespace is not None
            else cls._class.class_name
        )
        cls.template = cpptype(typename)

    def ctor(self, **kwargs) -> AugExpr:
        for candidate_ctor in self._class.constructors():
            ctor_argnames = [param.name for param in candidate_ctor.parameters]
            if set(ctor_argnames) == set(kwargs.keys()):
                break
        else:
            raise Exception(
                f"No constructor of class {self._class.class_name} matches the argument names {kwargs.keys()}"
            )

        ctor_args = [kwargs[name] for name in ctor_argnames]
        return self.ctor_bind(*ctor_args)
