import sympy as sp

import inspect

from pystencils import TypedSymbol, DynamicType, tcast
from pystencils.defaults import DEFAULTS


def expandable(name: str):
    def wrap(expansion_func):
        nargs = len(inspect.signature(expansion_func).parameters)
        return sp.Function(
            name, nargs=nargs, expansion_func=staticmethod(expansion_func)
        )

    return wrap


class DomainCoordinates:
    aabb_min = tuple(
        TypedSymbol(f"blockforest_aabb_min_{i}", DynamicType.NUMERIC_TYPE)
        for i in range(3)
    )

    aabb_max = tuple(
        TypedSymbol(f"blockforest_aabb_max_{i}", DynamicType.NUMERIC_TYPE)
        for i in range(3)
    )

    @expandable("domain.x_min")
    def x_min():
        return DomainCoordinates.aabb_min[0]

    @expandable("domain.y_min")
    def y_min():
        return DomainCoordinates.aabb_min[1]

    @expandable("domain.z_min")
    def z_min():
        return DomainCoordinates.aabb_min[2]

    @expandable("domain.x_max")
    def x_max():
        return DomainCoordinates.aabb_max[0]

    @expandable("domain.y_max")
    def y_max():
        return DomainCoordinates.aabb_max[1]

    @expandable("domain.z_max")
    def z_max():
        return DomainCoordinates.aabb_max[2]


domain = DomainCoordinates


class BlockCoordinates:
    aabb_min = tuple(
        TypedSymbol(f"block_aabb_min_{i}", DynamicType.NUMERIC_TYPE) for i in range(3)
    )
    aabb_max = tuple(
        TypedSymbol(f"block_aabb_max_{i}", DynamicType.NUMERIC_TYPE) for i in range(3)
    )
    ci_min = tuple(
        TypedSymbol(f"cell_interval_min_{i}", DynamicType.INDEX_TYPE) for i in range(3)
    )
    ci_max = tuple(
        TypedSymbol(f"cell_interval_max_{i}", DynamicType.INDEX_TYPE) for i in range(3)
    )


block = BlockCoordinates


class CellCoordinates:
    cell_extents = tuple(
        TypedSymbol(f"cell_extents_{i}", DynamicType.NUMERIC_TYPE) for i in range(3)
    )

    @expandable("cell.x")
    def x():
        return BlockCoordinates.aabb_min[0] + CellCoordinates.cell_extents[
            0
        ] * tcast.as_numeric(
            block.ci_min[0] + sp.Symbol(DEFAULTS.spatial_counter_names[0])
        )

    @expandable("cell.y")
    def y():
        return BlockCoordinates.aabb_min[1] + CellCoordinates.cell_extents[
            1
        ] * tcast.as_numeric(
            block.ci_min[1] + sp.Symbol(DEFAULTS.spatial_counter_names[1])
        )

    @expandable("cell.z")
    def z():
        return BlockCoordinates.aabb_min[2] + CellCoordinates.cell_extents[
            2
        ] * tcast.as_numeric(
            block.ci_min[2] + sp.Symbol(DEFAULTS.spatial_counter_names[2])
        )

    @expandable("cell.local_x")
    def local_x():
        return CellCoordinates.cell_extents[0] * tcast.as_numeric(
            block.ci_min[0] + DEFAULTS.spatial_counters[0]
        )

    @expandable("cell.local_y")
    def local_y():
        return CellCoordinates.cell_extents[1] * tcast.as_numeric(
            block.ci_min[1] + DEFAULTS.spatial_counters[1]
        )

    @expandable("cell.local_z")
    def local_z():
        return CellCoordinates.cell_extents[2] * tcast.as_numeric(
            block.ci_min[2] + DEFAULTS.spatial_counters[2]
        )

    @expandable("cell.dx")
    def dx():
        return CellCoordinates.cell_extents[0]

    @expandable("cell.dy")
    def dy():
        return CellCoordinates.cell_extents[1]

    @expandable("cell.dz")
    def dz():
        return CellCoordinates.cell_extents[2]


cell = CellCoordinates
