import sympy as sp
from pystencilssfg import SourceFileGenerator

from pystencils import fields, Field
from lbmpy import LBStencil, Stencil, LBMConfig, LBMOptimisation, create_lb_update_rule
from lbmpy.macroscopic_value_kernels import macroscopic_values_setter


from sfg_walberla import Sweep
from sfg_walberla.boundaries import FreeSlip

with SourceFileGenerator() as sfg:
    sfg.namespace("TestIrregularFreeSlip::gen")

    stencil = LBStencil(Stencil.D3Q19)
    d, q = stencil.D, stencil.Q
    f: Field
    f_tmp: Field
    f, f_tmp, rho, u = fields(
        f"f({q}), f_tmp({q}), rho, u({d}): double[{d}D]", layout="fzyx"
    )  # type: ignore

    stencil_name = stencil.name
    sfg.include(f"stencil/{stencil_name}.h")
    sfg.code(f"using LbStencil = walberla::stencil::{stencil_name};")

    lbm_config = LBMConfig(
        stencil=stencil,
        output={"density": rho, "velocity": u},
    )
    lbm_opt = LBMOptimisation(
        symbolic_field=f,
        symbolic_temporary_field=f_tmp,
    )

    lb_update = create_lb_update_rule(lbm_config=lbm_config, lbm_optimisation=lbm_opt)
    assert lb_update is not None

    lb_update_sweep = Sweep("LbStreamCollide", lb_update)
    lb_update_sweep.swap_fields(f, f_tmp)
    sfg.generate(lb_update_sweep)

    lb_init = macroscopic_values_setter(
        lb_update.method,
        density=sp.Symbol("density"),
        velocity=u.center_vector,
        pdfs=f,
    )
    lb_init_sweep = Sweep("LbInit", lb_init)
    sfg.generate(lb_init_sweep)

    irreg_freeslip = FreeSlip("FreeSlipIrregular", lb_update.method, f, wall_normal=FreeSlip.IRREGULAR)
    sfg.generate(irreg_freeslip)

