import sympy as sp

from pystencils import fields, kernel

from pystencilssfg import SourceFileGenerator, SfgConfiguration
from sfg_walberla import Sweep

cfg = SfgConfiguration(output_directory="out")
with SourceFileGenerator(cfg) as sfg:
    u_src, u_dst, f = fields("u_src, u_dst, f(1) : double[2D]", layout="fzyx")
    h = sp.Symbol("h")

    @kernel
    def poisson_jacobi():
        u_dst[0,0] @= (h**2 * f[0, 0] + u_src[1, 0] + u_src[-1, 0] + u_src[0, 1] + u_src[0, -1]) / 4

    sfg.generate(Sweep("Jacobi", poisson_jacobi))
