# Managing the Code Generator's Python Environment

On this page, you can find information on managing, customizing, and extending the Python environment
used by the waLBerla code generation system.

## Using the Private Virtual Environment

By default, `sfg-walberla` creates a new Python virtual environment within the CMake build tree,
and there installs all packages required for code generation.
This can be disabled by setting the `WALBERLA_CODEGEN_PRIVATE_VENV` CMake cache variable to `FALSE`.

### Install Additional Packages

For projects that require external dependencies, *sfg-walberla* exposes the CMake function
`walberla_codegen_venv_install`, which can be used to install additional packages into the
code generator virtual environment;
for instance, the following invocation installs `pyyaml`:

```CMake
walberla_codegen_venv_install( pyyaml )
```

The arguments passed to `walberla_codegen_venv_install` are forwarded directly to `pip install`.
You can therefore use any parameters that `pip` can interpret, for instance `-e` to perform an
editable install, or `-r <requirements-file>` to install packages from a requirements file.

## Using an External Virtual Environment

If `WALBERLA_CODEGEN_PRIVATE_VENV` is set to `FALSE`, sfg-walberla will use the Python interpreter
found in the CMake environment for running the code generators.
You can customize your Python interpreter by setting the `Python_EXECUTABLE` or `Python_ROOT_DIR` hints.

:::{seealso}
[FindPython CMake Module](https://cmake.org/cmake/help/latest/module/FindPython.html)
:::

Sfg-walberla will check if the required packages are installed into the given external Python environment,
and raise an error if any are missing.
